import os

from pymap3d.ellipsoid import Ellipsoid

from magcoordpy.igrf_coeffs import load_coeffs

igrf_file = os.path.dirname(os.path.abspath(__file__)) + "/data/igrf13coeffs.txt"
GH = load_coeffs(igrf_file)
# The World Geodetic System 1984 (WGS84) datum
earth_ellipsoid = Ellipsoid("wgs84")
# equatorial radius of ellipsoid
RE_M = earth_ellipsoid.semimajor_axis
# eccentricity squared of the ellipsoid
E_SQ = earth_ellipsoid.eccentricity ** 2



import logging

import numpy as np

logger = logging.getLogger("igrf-coeffs")


def load_coeffs(filename):
    """
    Load igrf12 coeffs from file.
    Adapted from pyIGRF (https://github.com/zzyztyy/pyIGRF/blob/master/pyIGRF/loadCoeffs.py)

    Parameters
    ----------
    filename: str
        file which save IGRF coeffs

    Returns
    -------
    list : list[float]
        g and h list one by one
    """
    gh = []
    gh2arr = []
    with open(filename) as f:
        text = f.readlines()
        for a in text:
            if a[:2] == "g " or a[:2] == "h ":
                b = a.split()[3:]
                b = [float(x) for x in b]
                gh2arr.append(b)
        gh2arr = np.array(gh2arr).transpose()
        N = len(gh2arr)
        for i in range(N):
            if i < 19:
                for j in range(120):
                    gh.append(gh2arr[i][j])
            else:
                for p in gh2arr[i]:
                    gh.append(p)
        gh.append(0)
        return gh


def get_coeffs(gh, date):
    """
    Get IGRF Gauss coefficient for a given date.
    Adapted from pyIGRF (https://github.com/zzyztyy/pyIGRF/blob/master/pyIGRF/loadCoeffs.py)

    Parameters
    ----------
    gh : list
        from load_coeffs
    date : float

    Returns
    -------
    tuple : tuple[list, list]
    """
    if date < 1900.0 or date > 2030.0:
        logger.error("This subroutine will not work with a date of " + str(date))
        logger.error("Date must be in the range 1900.0 <= date <= 2030.0")
        logger.error("On return [], []")
        return [], []

    elif date >= 2020.0:
        if date > 2025.0:
            # not adapt for the model but can calculate
            logger.warning("This version of the IGRF is intended for use up to 2025.0.")
            logger.warning(
                "values for "
                + str(date)
                + " will be computed but may be of reduced accuracy"
            )
        t = date - 2020.0
        tc = 1.0
        # pointer for last coefficient in pen-ultimate set of MF coefficients...
        ll = 3060 + 195
        nmx = 13
        nc = nmx * (nmx + 2)
    else:
        t = 0.2 * (date - 1900.0)
        ll = int(t)
        t = t - ll
        # SH models before 1995.0 are only to degree 10
        if date < 1995.0:
            nmx = 10
            nc = nmx * (nmx + 2)
            ll = nc * ll
        else:
            nmx = 13
            nc = nmx * (nmx + 2)
            ll = int(0.2 * (date - 1995.0))
            # 19 is the number of SH models that extend to degree 10
            ll = 120 * 19 + nc * ll
        tc = 1.0 - t

    g, h = [], []
    temp = ll - 1
    for n in range(nmx + 1):
        g.append([])
        h.append([])
        if n == 0:
            g[0].append(None)
        for m in range(n + 1):
            if m != 0:
                g[n].append(tc * gh[temp] + t * gh[temp + nc])
                h[n].append(tc * gh[temp + 1] + t * gh[temp + nc + 1])
                temp += 2

            else:
                g[n].append(tc * gh[temp] + t * gh[temp + nc])
                h[n].append(None)
                temp += 1

    return g, h



import datetime as dt

import numpy as np


def subsol(datetime):
    """
    Finds subsolar geocentric latitude and longitude.
    Author: adapted from apexpy.

    Parameters
    ----------
    datetime : :class:`datetime.datetime` or :class:`numpy.ndarray[datetime64]`
        Date and time in UTC (naive objects are treated as UTC)
    Returns
    -------
    sbsllat : float
        Latitude of subsolar point
    sbsllon : float
        Longitude of subsolar point

    Notes
    -----
    Based on formulas in Astronomical Almanac for the year 1996, p. C24.
    (U.S. Government Printing Office, 1994). Usable for years 1601-2100,
    inclusive. According to the Almanac, results are good to at least 0.01
    degree latitude and 0.025 degrees longitude between years 1950 and 2050.
    Accuracy for other years has not been tested. Every day is assumed to have
    exactly 86400 seconds; thus leap seconds that sometimes occur on December
    31 are ignored (their effect is below the accuracy threshold of the
    algorithm).
    After Fortran code by A. D. Richmond, NCAR. Translated from IDL
    by K. Laundal.
    """
    # Convert to year, day of year and seconds since midnight
    if isinstance(datetime, dt.datetime):
        year = np.asanyarray([datetime.year])
        doy = np.asanyarray([datetime.timetuple().tm_yday])
        ut = np.asanyarray(
            [datetime.hour * 3600 + datetime.minute * 60 + datetime.second]
        )
    elif isinstance(datetime, np.ndarray):
        # This conversion works for datetime of wrong precision or unit epoch
        times = datetime.astype("datetime64[s]")
        year_floor = times.astype("datetime64[Y]")
        day_floor = times.astype("datetime64[D]")
        year = year_floor.astype(int) + 1970
        doy = (day_floor - year_floor).astype(int) + 1
        ut = (times.astype("datetime64[s]") - day_floor).astype(float)
    else:
        raise ValueError("input must be datetime.datetime or numpy array")

    if not (np.all(1601 <= year) and np.all(year <= 2100)):
        raise ValueError("Year must be in [1601, 2100]")

    yr = year - 2000

    nleap = np.floor((year - 1601.0) / 4.0).astype(int)
    nleap -= 99
    mask_1900 = year <= 1900
    if np.any(mask_1900):
        ncent = np.floor((year[mask_1900] - 1601.0) / 100.0).astype(int)
        ncent = 3 - ncent
        nleap[mask_1900] = nleap[mask_1900] + ncent

    l0 = -79.549 + (-0.238699 * (yr - 4.0 * nleap) + 3.08514e-2 * nleap)
    g0 = -2.472 + (-0.2558905 * (yr - 4.0 * nleap) - 3.79617e-2 * nleap)

    # Days (including fraction) since 12 UT on January 1 of IYR:
    df = (ut / 86400.0 - 1.5) + doy

    # Mean longitude of Sun:
    lmean = l0 + 0.9856474 * df

    # Mean anomaly in radians:
    grad = np.radians(g0 + 0.9856003 * df)

    # Ecliptic longitude:
    lmrad = np.radians(lmean + 1.915 * np.sin(grad) + 0.020 * np.sin(2.0 * grad))
    sinlm = np.sin(lmrad)

    # Obliquity of ecliptic in radians:
    epsrad = np.radians(23.439 - 4e-7 * (df + 365 * yr + nleap))

    # Right ascension:
    alpha = np.degrees(np.arctan2(np.cos(epsrad) * sinlm, np.cos(lmrad)))

    # Declination, which is also the subsolar latitude:
    sslat = np.degrees(np.arcsin(np.sin(epsrad) * sinlm))

    # Equation of time (degrees):
    etdeg = lmean - alpha
    nrot = np.round(etdeg / 360.0)
    etdeg = etdeg - 360.0 * nrot

    # Subsolar longitude calculation. Earth rotates one degree every 240 s.
    sslon = 180.0 - (ut / 240.0 + etdeg)
    nrot = np.round(sslon / 360.0)
    sslon = sslon - 360.0 * nrot

    # Return a single value from the output if the input was a single value
    if isinstance(datetime, dt.datetime):
        return sslat[0], sslon[0]
    return sslat, sslon



import logging
from math import pi

import numpy as np
import pymap3d

import magcoordpy.constants as CONSTS
import magcoordpy.helpers as helpers
import magcoordpy.igrf_coeffs as igrf_coeffs

logger = logging.getLogger("spwx-coord")


def geodetic2ecef(geod_lati_deg, geod_long_deg, geod_alti_m):
    """Conversion from Geodetic (lat, lon, alt) to geocentric Cartesian (x, y, z) coordinates.

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)

    Parameters
    ----------
    geod_long_deg : np.ndarray
        array containing geodetic longitudes in degrees
    geod_lati_deg : np.ndarray
        array containing geodetic longitudes in degrees
    geod_alti_m : np.ndarray
        array containing geodetic altitude in meters

    Returns
    -------
    tuple[np.ndarray, np.ndarray, np.ndarray]
    """
    geod_long = np.deg2rad(geod_long_deg)
    geod_lati = np.deg2rad(geod_lati_deg)

    rho = CONSTS.RE_M * (1 - CONSTS.E_SQ * (np.sin(geod_lati) ** 2)) ** (-0.5)

    x = (rho + geod_alti_m) * np.cos(geod_lati) * np.cos(geod_long)
    y = (rho + geod_alti_m) * np.cos(geod_lati) * np.sin(geod_long)
    z = (rho + geod_alti_m - CONSTS.E_SQ * rho) * np.sin(geod_lati)

    return x, y, z


def compute_pos_centered_dipole_north_pole(year=2021.0):
    """
    Function that returns the position of the centered dipole (CD) northern pole using first three Gauss coefficients.
    Coordinates of the CD northern pole are returned in Geocentric Spherical (colat, long) coordinates [rad].

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)


    Parameters
    ----------
    year : float, default=2021.0
        year for the computation

    Returns
    -------
    tuple[float, float]
    """
    g, h = igrf_coeffs.get_coeffs(CONSTS.GH, year)

    if len(g) == 0:
        raise ValueError(" IGRF coefficients not correctly extracted")

    # first three Gauss coefficients (n = 1)
    g_n1_m0 = g[1][0]
    g_n1_m1 = g[1][1]
    h_n1_m1 = h[1][1]

    b0 = (g_n1_m0 ** 2 + g_n1_m1 ** 2 + h_n1_m1 ** 2) ** 0.5

    logger.debug(f" B0 = {b0}")

    # geocentric colatitude of the CD pole coordinates in NH
    geocentric_colat_n = np.arccos(-g_n1_m0 / b0)

    # geocentric longitude of the CD pole coordinates in NH
    geocentric_long_n = np.arctan2(h_n1_m1, g_n1_m1) - pi

    logger.info(
        f" Geocentric coordinates of the CD pole coordinates in {year}:"
        f" Colatitude: {np.round(np.rad2deg(geocentric_colat_n), 2)}"
        f" Longitude: {np.round(np.rad2deg(geocentric_long_n), 2)}"
    )

    return geocentric_colat_n, geocentric_long_n


def mlon2mlt(mlon_cd, datetime, ssheight=50 * 6371):
    """Computes the magnetic local time at the specified magnetic longitude and UT.

    Author: Giorgio Savastano (giorgiosavastano@gmail.com) adapted from apexpy.

    Parameters
    ----------
    mlon_cd : array_like
        Centered Dipole (CD) magnetic longitude
    datetime : :class:`datetime.datetime`
        Date and time
    ssheight : float, optional
        Altitude in km to use for converting the subsolar point from
        geographic to magnetic coordinates. A high altitude is used
        to ensure the subsolar point is mapped to high latitudes, which
        prevents the South-Atlantic Anomaly (SAA) from influencing the MLT.
    Returns
    -------
    ndarray or float
        Magnetic local time [0, 24)
    Notes
    -----
    To compute the MLT, we find the apex longitude of the subsolar point at
    the given time. Then the MLT of the given point will be computed from
    the separation in magnetic longitude from this point (1 hour = 15
    degrees).
    """
    sslat_geo, sslon_geo = helpers.subsol(datetime)
    sslat_cd, sslon_cd, ssalt_cd = geodetic2cd(
        float(sslat_geo), float(sslon_geo), float(ssheight)
    )

    # np.float64 will ensure lists are converted to arrays
    return (180 + np.float64(mlon_cd) - sslon_cd) / 15 % 24


def ecef2eccdf(x_geoc, y_geoc, z_geoc, year=2021.0):
    """Transformation from Geocentric Cartesian (x, y, z) to Centered Dipole (CD) Cartesian (x, y, z) coordinates.

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)

    Parameters
    ----------
    x_geoc : np.ndarray
        array containing x coordinates
    y_geoc : np.ndarray
        array containing y coordinates
    z_geoc : np.ndarray
        array containing z coordinates
    year : float, default=2021.0
        year for computing the IGRF Gaus coefficients

    Returns
    -------
    tuple : tuple[np.ndarray, np.ndarray, np.ndarray]
    """
    if type(x_geoc) == list:
        logger.info(" Converting list to np.ndarrays.")
        x_geoc = np.asarray(x_geoc)
        y_geoc = np.asarray(y_geoc)
        z_geoc = np.asarray(z_geoc)
    elif type(x_geoc) != np.ndarray:
        logger.info(f" Converting {type(x_geoc)} to np.ndarrays.")
        x_geoc = np.asarray([x_geoc])
        y_geoc = np.asarray([y_geoc])
        z_geoc = np.asarray([z_geoc])

    # position of the centered dipole (CD) northern pole
    geoc_colat_n, geoc_long_n = compute_pos_centered_dipole_north_pole(year=year)

    # Compute the base vectors of the CD coordinate system
    # the transpose CD base vectors form the rotation matrix

    # m = unit vector anti-parallel to the dipole axis
    # this correspond to the z axis of the CD coordinate system
    # (Cartesian z axis aligns with the dipole axis)
    m = np.asarray(
        [
            np.sin(geoc_colat_n) * np.cos(geoc_long_n),
            np.sin(geoc_colat_n) * np.sin(geoc_long_n),
            np.cos(geoc_colat_n),
        ]
    )

    # x axis of the CD coordinate system
    xx = np.asarray(
        [
            np.cos(geoc_colat_n) * np.cos(geoc_long_n),
            np.cos(geoc_colat_n) * np.sin(geoc_long_n),
            -np.sin(geoc_colat_n),
        ]
    )

    # y axis of the CD coordinate system
    yy = np.asarray([-np.sin(geoc_long_n), np.cos(geoc_long_n), 0])
    # Rotation matrix
    R_geoc_cd = np.asarray([xx, yy, m])

    matrix_arrays = np.asarray([x_geoc, y_geoc, z_geoc]).T

    # multiplication between the rotation matrix and the array of vectors
    # I had to invert the order of the multiplication to take advantage of numpy vectorization
    x_cd_arr, y_cd_arr, z_cd_arr = (matrix_arrays @ R_geoc_cd.T).T

    return x_cd_arr, y_cd_arr, z_cd_arr


def ecef2spherical(x, y, z, decimals=2):
    """Transformation from geocentric Cartesian (x, y, z) to Spherical (colat, long, r) coordinates.

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)

    Parameters
    ----------
    x : np.ndarray
        array containing x coordinates
    y : np.ndarray
        array containing y coordinates
    z : np.ndarray
        array containing z coordinates
    decimals : int, default=2
            Number of decimal places to round to.  If
            decimals is negative, it specifies the number of positions to
            the left of the decimal point

    Returns
    -------
    tuple[np.ndarray, np.ndarray, np.ndarray]
    """
    # geocentric radial distance 𝑟
    r = np.sqrt(x * x + y * y + z * z)
    # the colatitude 𝜃
    colat = np.round(np.rad2deg(np.arccos(z / r)), decimals)
    # the east longitude 𝜙
    long = np.round(np.rad2deg(np.arctan2(y, x)), decimals)

    return colat, long, np.round(r, decimals)


def spherical2ecef(colat_geoc_arr, long_geoc_arr, radial_dist_geoc_arr):
    """Transformation from Geocentric (GEO) Spherical (colat, lon, r) to Cartesian (x,y,z).

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)

    Parameters
    ----------
    colat_geoc_arr : np.ndarray
        array containing colatitude component of Geocentric coordinates in rad
    long_geoc_arr : np.ndarray
        array containing longitude component of Geocentric coordinates in rad
    radial_dist_geoc_arr : np.ndarray
        array containing radial distance component of Geocentric coordinates in m

    Returns
    -------

    """
    x_geoc_arr = radial_dist_geoc_arr * np.sin(colat_geoc_arr) * np.cos(long_geoc_arr)
    y_geoc_arr = radial_dist_geoc_arr * np.sin(colat_geoc_arr) * np.sin(long_geoc_arr)
    z_geoc_arr = radial_dist_geoc_arr * np.cos(colat_geoc_arr)

    return x_geoc_arr, y_geoc_arr, z_geoc_arr


def geodetic2cd(
    gglat_deg_array, gglon_deg_array, ggalt_km_array, decimals=2, year=2021.0
):
    """Transformation from Geodetic (lat, lon, alt) to Centered Dipole (CD) (lat, lon, alt).

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)

    Parameters
    ----------
    gglon_deg_array : np.ndarray
        array containing geodetic longitude values in degrees
    gglat_deg_array : np.ndarray
        array containing geodetic latitude values in degrees
    ggalt_km_array : np.ndarray
        array containing geodetic altitude values in km
    decimals : int, default=2
        Number of decimal places to round to. If
        decimals is negative, it specifies the number of positions to
        the left of the decimal point.
    year : float, default=2021.0
        year for computing the IGRF Gauss coefficients

    Returns
    -------
    tuple[np.ndarray, np.ndarray, np.ndarray]
        CD lat, lon, alt arrays
    """
    if type(gglon_deg_array) == list:
        logger.info(" Converting list to np.ndarrays.")
        gglon_deg_array = np.asarray(gglon_deg_array)
        gglat_deg_array = np.asarray(gglat_deg_array)
        ggalt_km_array = np.asarray(ggalt_km_array)
    elif type(gglon_deg_array) != np.ndarray:
        logger.info(f" Converting {type(gglon_deg_array)} to np.ndarrays.")
        gglon_deg_array = np.asarray([gglon_deg_array])
        gglat_deg_array = np.asarray([gglat_deg_array])
        ggalt_km_array = np.asarray([ggalt_km_array])

    x_geoc, y_geoc, z_geoc = pymap3d.geodetic2ecef(
        gglat_deg_array, gglon_deg_array, ggalt_km_array * 1000.0
    )

    x_cd, y_cd, z_cd = ecef2eccdf(x_geoc, y_geoc, z_geoc, year=year)

    colat_cd, long_cd, r_cd = ecef2spherical(x_cd, y_cd, z_cd)
    lat_cd = np.round(90 - colat_cd, decimals)
    alt_cd = np.round(r_cd - CONSTS.RE_M, decimals)

    return lat_cd, long_cd, alt_cd


def eccdf2ecef(x_cd, y_cd, z_cd, year=2021.0):
    """Transformation from Centered Dipole (CD) Cartesian (x, y, z) to Geocentric Cartesian (x, y, z) coordinates.

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)

    Parameters
    ----------
    x_cd : np.ndarray
        array containing x component of CD coordinates
    y_cd : np.ndarray
        array containing y component of CD coordinates
    z_cd : np.ndarray
        array containing z component of CD coordinates
    year : float, default=2021.0
        year for computing the IGRF Gaus coefficients

    Returns
    -------
    tuple[np.ndarray, np.ndarray, np.ndarray]
    """
    if type(x_cd) == list:
        logger.info(" Converting list to np.ndarrays.")
        x_cd = np.asarray(x_cd)
        y_cd = np.asarray(y_cd)
        z_cd = np.asarray(z_cd)
    elif type(x_cd) != np.ndarray:
        logger.info(f" Converting {type(x_cd)} to np.ndarrays.")
        x_cd = np.asarray([x_cd])
        y_cd = np.asarray([y_cd])
        z_cd = np.asarray([z_cd])

    # position of the centered dipole (CD) northern pole
    geoc_colat_n, geoc_long_n = compute_pos_centered_dipole_north_pole(year=year)

    # Compute the base vectors of the CD coordinate system
    # the transpose CD base vectors form the rotation matrix

    # m = unit vector anti-parallel to the dipole axis
    # this correspond to the z axis of the CD coordinate system
    # (Cartesian z axis aligns with the dipole axis)
    m = np.asarray(
        [
            np.sin(geoc_colat_n) * np.cos(geoc_long_n),
            np.sin(geoc_colat_n) * np.sin(geoc_long_n),
            np.cos(geoc_colat_n),
        ]
    )

    # x axis of the CD coordinate system
    xx = np.asarray(
        [
            np.cos(geoc_colat_n) * np.cos(geoc_long_n),
            np.cos(geoc_colat_n) * np.sin(geoc_long_n),
            -np.sin(geoc_colat_n),
        ]
    )

    # y axis of the CD coordinate system
    yy = np.asarray([-np.sin(geoc_long_n), np.cos(geoc_long_n), 0])
    # Rotation matrix is the transpose since the vectors are orthonormal
    R_cd_geoc = np.asarray([xx, yy, m]).T

    matrix_arrays = np.asarray([x_cd, y_cd, z_cd]).T

    # multiplication between the rotation matrix and the array of vectors
    # I had to invert the order of the multiplication to take advantage of numpy vectorization
    x_geoc_arr, y_geoc_arr, z_geoc_arr = (matrix_arrays @ R_cd_geoc.T).T

    return x_geoc_arr, y_geoc_arr, z_geoc_arr


def cd2geodetic(lat_cd_arr, lon_cd_arr, alt_cd_arr, decimals=3, year=2021.0):
    """Transformation from Centered Dipole (CD) (lat, lon, alt) to Geodetic (lat, lon, alt).

    Author: Giorgio Savastano (giorgiosavastano@gmail.com)

    Parameters
    ----------
    lat_cd_arr : np.ndarray
        array containing latitude component of CD coordinates in degrees
    lon_cd_arr : np.ndarray
        array containing longitude component of CD coordinates in degrees
    alt_cd_arr : np.ndarray
        array containing altitude (N.B. not the radial distance)
        component of CD coordinates in meters
    decimals : int, default=3
        Number of decimal places to round to. If
        decimals is negative, it specifies the number of positions to
        the left of the decimal point
    year : float, default=2021.0
        year for computing the IGRF Gauss coefficients

    Returns
    -------
    tuple[np.ndarray, np.ndarray, np.ndarray]
    """
    # convert lat to colat
    colat_cd_arr = 90 - lat_cd_arr
    rad_distance = alt_cd_arr + CONSTS.RE_M

    x_cd_arr, y_cd_arr, z_cd_arr = spherical2ecef(
        np.deg2rad(colat_cd_arr), np.deg2rad(lon_cd_arr), rad_distance
    )

    x_geoc_arr, y_geoc_arr, z_geoc_arr = eccdf2ecef(
        x_cd_arr, y_cd_arr, z_cd_arr, year=year
    )

    lat_geoc, long_geoc, alt_geoc = pymap3d.ecef2geodetic(
        x_geoc_arr, y_geoc_arr, z_geoc_arr
    )

    return (
        np.round(lat_geoc, decimals),
        np.round(long_geoc, decimals),
        np.round(alt_geoc, decimals),
    )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['magcoordpy']

package_data = \
{'': ['*'], 'magcoordpy': ['data/*']}

install_requires = \
['numpy>=1.22.2,<2.0.0', 'pymap3d>=2.7.2,<3.0.0', 'urllib3>=1.26.8,<2.0.0']

setup_kwargs = {
    'name': 'magcoordpy',
    'version': '0.2.4',
    'description': 'A python package for working with magnetic coordinate transformations',
    'long_description': '# MagCoordPy\n\n![test-main](https://github.com/giorgiosavastano/magcoordpy/actions/workflows/python-test-main.yml/badge.svg)\n![coverage-main](https://img.shields.io/codecov/c/github/giorgiosavastano/magcoordpy)\n![license](https://img.shields.io/github/license/giorgiosavastano/magcoordpy)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/magcoordpy)\n\nA python package for working with magnetic coordinate transformations.\nThe documentation is available at <https://magcoordpy.readthedocs.io/en/latest/>.\n\nInstallation\n------------\n\n    pip install magcoordpy\n\nExample usage\n-------------\n\n    from magcoordpy import coord_transforms\n    long_geo = np.arange(-180, 190, 10)\n    lati_geo = np.zeros(len(long_array))\n    alti_geo = np.zeros(len(long_array))\n    lat_cd, lon_cd, r_cd = coord_transforms.geodetic2cd(lati_geo, long_geo, alti_geo, year=2021.0)\n\n\nIt includes the following functions (not exhaustive list):\n\n* geodetic2cd --> transformation from geodetic to centered dipole\n* cd2geodetic --> transformation from centered dipole to geodetic\n\n\n## Authors\n\n- Giorgio Savastano (<giorgiosavastano@gmail.com>)\n\nPlease use github issues to make bug reports and request new functionality. Contributions are always welcome.\n\n## References\n\nLaundal, K.M., Richmond, A.D. Magnetic Coordinate Systems. Space Sci Rev 206, 27–59 (2017). <https://doi.org/10.1007/s11214-016-0275-y>\n',
    'author': 'Giorgio Savastano',
    'author_email': 'giorgio.savastano@uniroma1.it',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
